@@ -62,31 +62,31 @@ correctly, as this form of method chaining is not typical in Python.
 
 from __future__ import annotations
 
-import json
 import logging
 import warnings
-from abc import ABC
 from pathlib import Path
 from typing import (
     TYPE_CHECKING,
-    Any,
     Callable,
-    Dict,
-    Generator,
-    List,
     Literal,
-    Set,
     overload,
 )
 
 from yarl import URL
 
 import pact.v3.ffi
+from pact.v3.error import (
+    InteractionVerificationError,
+    Mismatch,
+    MismatchesError,
+    PactVerificationError,
+)
 from pact.v3.interaction._async_message_interaction import AsyncMessageInteraction
 from pact.v3.interaction._http_interaction import HttpInteraction
 from pact.v3.interaction._sync_message_interaction import SyncMessageInteraction
 
 if TYPE_CHECKING:
+    from collections.abc import Generator
     from types import TracebackType
 
     from pact.v3.interaction import Interaction
@@ -99,120 +99,6 @@ if TYPE_CHECKING:
 logger = logging.getLogger(__name__)
 
 
-class PactError(Exception, ABC):
-    """
-    Base class for exceptions raised by the Pact module.
-    """
-
-
-class InteractionVerificationError(PactError):
-    """
-    Exception raised due during the verification of an interaction.
-
-    This error is raised when an error occurs during the manual verification of an
-    interaction. This is typically raised when the consumer fails to handle the
-    interaction correctly thereby generating its own exception. The cause of the
-    error is stored in the `error` attribute.
-    """
-
-    def __init__(self, description: str, error: Exception) -> None:
-        """
-        Initialise a new InteractionVerificationError.
-
-        Args:
-            description:
-                Description of the interaction that failed verification.
-
-            error: Error that occurred during the verification of the
-                interaction.
-        """
-        super().__init__(f"Error verifying interaction '{description}': {error}")
-        self._description = description
-        self._error = error
-
-    @property
-    def description(self) -> str:
-        """
-        Description of the interaction that failed verification.
-        """
-        return self._description
-
-    @property
-    def error(self) -> Exception:
-        """
-        Error that occurred during the verification of the interaction.
-        """
-        return self._error
-
-
-class PactVerificationError(PactError):
-    """
-    Exception raised due to errors in the verification of a Pact.
-
-    This is raised when performing manual verification of the Pact through the
-    [`verify`][pact.v3.Pact.verify] method:
-
-    ```python
-    pact = Pact("consumer", "provider")
-    # Define interactions...
-    try:
-        pact.verify(handler, kind="Async")
-    except PactVerificationError as e:
-        print(e.errors)
-    ```
-
-    All of the errors that occurred during the verification of all of the
-    interactions are stored in the `errors` attribute.
-
-    This is different from the [`MismatchesError`][pact.v3.MismatchesError]
-    which is raised when there are mismatches detected by the mock server.
-    """
-
-    def __init__(self, errors: list[InteractionVerificationError]) -> None:
-        """
-        Initialise a new PactVerificationError.
-
-        Args:
-            errors:
-                Errors that occurred during the verification of the Pact.
-        """
-        super().__init__(f"Error verifying Pact (count: {len(errors)})")
-        self._errors = errors
-
-    @property
-    def errors(self) -> list[InteractionVerificationError]:
-        """
-        Errors that occurred during the verification of the Pact.
-        """
-        return self._errors
-
-
-class MismatchesError(PactError):
-    """
-    Exception raised when there are mismatches between the Pact and the server.
-    """
-
-    def __init__(self, mismatches: list[dict[str, Any]]) -> None:
-        """
-        Initialise a new MismatchesError.
-
-        Args:
-            mismatches:
-                Mismatches between the Pact and the server.
-        """
-        super().__init__(f"Mismatched interaction (count: {len(mismatches)})")
-        self._mismatches = mismatches
-
-    # TODO: Replace the list of dicts with a more structured object.
-    # https://github.com/pact-foundation/pact-python/issues/644
-    @property
-    def mismatches(self) -> list[dict[str, Any]]:
-        """
-        Mismatches between the Pact and the server.
-        """
-        return self._mismatches
-
-
 class Pact:
     """
     A Pact between a consumer and a provider.
@@ -256,7 +142,7 @@ class Pact:
 
         self._consumer = consumer
         self._provider = provider
-        self._interactions: Set[Interaction] = set()
+        self._interactions: set[Interaction] = set()
         self._handle: pact.v3.ffi.PactHandle = pact.v3.ffi.new_pact(
             consumer,
             provider,
@@ -520,7 +406,7 @@ class Pact:
     @overload
     def verify(
         self,
-        handler: Callable[[str | bytes | None, Dict[str, str]], None],
+        handler: Callable[[str | bytes | None, dict[str, str]], None],
         kind: Literal["Async", "Sync"],
         *,
         raises: Literal[True] = True,
@@ -528,19 +414,19 @@ class Pact:
     @overload
     def verify(
         self,
-        handler: Callable[[str | bytes | None, Dict[str, str]], None],
+        handler: Callable[[str | bytes | None, dict[str, str]], None],
         kind: Literal["Async", "Sync"],
         *,
         raises: Literal[False],
-    ) -> List[InteractionVerificationError]: ...
+    ) -> list[InteractionVerificationError]: ...
 
     def verify(
         self,
-        handler: Callable[[str | bytes | None, Dict[str, str]], None],
+        handler: Callable[[str | bytes | None, dict[str, str]], None],
         kind: Literal["Async", "Sync"],
         *,
         raises: bool = True,
-    ) -> List[InteractionVerificationError] | None:
+    ) -> list[InteractionVerificationError] | None:
         """
         Verify message interactions.
 
@@ -574,7 +460,7 @@ class Pact:
                 process a message. If set to `False`, then the function will
                 return a list of errors.
         """
-        errors: List[InteractionVerificationError] = []
+        errors: list[InteractionVerificationError] = []
         for message in self.interactions(kind):
             request: pact.v3.ffi.MessageContents | None = None
             if isinstance(message, pact.v3.ffi.SynchronousMessage):
@@ -774,7 +660,7 @@ class PactServer:
         return pact.v3.ffi.mock_server_matched(self._handle)
 
     @property
-    def mismatches(self) -> list[dict[str, Any]]:
+    def mismatches(self) -> list[Mismatch]:
         """
         Mismatches between the Pact and the server.
 
@@ -788,7 +674,12 @@ class PactServer:
         if not self._handle:
             msg = "The server is not running."
             raise RuntimeError(msg)
-        return pact.v3.ffi.mock_server_mismatches(self._handle)
+        return list(
+            map(
+                Mismatch.from_dict,
+                pact.v3.ffi.mock_server_mismatches(self._handle),
+            )
+        )
 
     @property
     def logs(self) -> str | None:
@@ -867,12 +758,13 @@ class PactServer:
         """
         if self._handle and not self.matched:
             if self._verbose:
-                logger.error(
-                    "Mismatches:\n%s",
-                    json.dumps(self.mismatches, indent=2),
-                )
+                msg = "\n".join([
+                    "Mismatches:",
+                    *(f"  ({i + 1}) {m}" for i, m in enumerate(self.mismatches)),
+                ])
+                logger.error(msg)
             if self._raises:
-                raise MismatchesError(self.mismatches)
+                raise MismatchesError(*self.mismatches)
             self._handle = None
 
     def __truediv__(self, other: str | object) -> URL:
